# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/superpower.anova.ipynb (unless otherwise specified).

__all__ = ['ANOVA', 'ANCOVA', 'ANCOVAMultiway']

# Cell
from ..rpy import RFunction

# Cell
class ANOVA(RFunction):

    argNames = ['labels_factors', 'labels_levels', 'num_factors', 'num_levels', 'factorTypes', 'means',
               'SDs', 'N', 'pVal', 'full_check', 'sims', 'test_that']

    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='powerANOVAS', argNames=argNames, **kwargs)

# Cell
class ANCOVA(RFunction):
    argNames = ['labels_factors', 'labels_levels', 'num_factors', 'num_levels', 'factorTypes', 'means', 'fixedVals',
               'SDs', 'N', 'pVal', 'full_check', 'means_covariates', 'SDs_covariates', 'fixedVals',
                'labels_covariates', 'sims']

    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='powerANCOVA', argNames=argNames, **kwargs)

# Cell
class ANCOVAMultiway(RFunction):

    argNames = ['labels_factors', 'labels_levels', 'num_factors', 'num_levels', 'factorTypes', 'means', 'fixedVals',
               'SDs', 'N', 'pVal', 'full_check', 'means_covariates', 'SDs_covariates', 'fixedVals',
                'labels_covariates', 'sims']

    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='powerANCOVA_multiway', argNames=argNames, **kwargs)