# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/superpower.ttest.ipynb (unless otherwise specified).

__all__ = ['TTest']

# Cell
from ..rpy import RFunction

# Cell
class TTest(RFunction):

    argNames = ['N', 'n1', 'n2', 'ratio', 'sims', 'meanDifference',
               'means1', 'sd1', 'means2', 'sd2', 'var_equal', 'type',
               'corr', 'altHypothesis', 'variances', 'pVal']

    def __init__(self, argNames=argNames, **kwargs):
        super().__init__(name='powerTtest', argNames=argNames, **kwargs)